==> game-hook-base-1.0-sources.jar: cn/leancloud/play/hook/context/AbstractOperationContext.java <==
package cn.leancloud.play.hook.context;

import cn.leancloud.play.hook.Reason;
import cn.leancloud.play.hook.request.AbstractRequest;
import cn.leancloud.play.hook.HookResponse;

import java.util.Objects;
import java.util.concurrent.CompletableFuture;

/**
 * 每个 Hook 被调用时都会传入一个 Context。Context 内封装着 Hook 的名称、触发 Hook 调用的玩家
 * 请求参数，以及当前 Hook 请求的处理状态。Hook 处理结果也是通过 Context 通知给 Game Server。可以将
 * Context 当做 Hook 与 Game Server 之间通信、交互的桥梁。
 *
 */
abstract class AbstractOperationContext<T extends AbstractRequest> {
    private final CompletableFuture<HookResponse<T>> future;
    private final T req;

    private ContextStatus status;

    AbstractOperationContext(T req, CompletableFuture<HookResponse<T>> future) {
        Objects.requireNonNull(req);
        Objects.requireNonNull(future);

        this.req = req;
        this.future = future;
        this.status = ContextStatus.NEW;
    }

    /**
     * 获取当前 Context 状态
     *
     * @return 返回 Context 状态
     */
    public ContextStatus getStatus() {
        return status;
    }

    /**
     * 获取与当前 Context 绑定的 Hook 请求参数实例
     *
     * @return 返回 Hook 请求参数实例
     */
    public T getRequest() {
        return req;
    }

    /**
     * 获取当前 Context 所属 Hook 名称
     *
     * @return 返回 Hook 名称
     */
    public abstract String getHookName();

    /**
     * 同意本次请求，Game Server 会执行 Hook 之后的操作
     */
    public void continueProcess() {
        updateStatusWhenNotProcessed(ContextStatus.CONTINUED);
        completeHookFuture(HookResponse.success(req));
    }

    /**
     * 拒绝本次请求，并发送 Reason 信息给发送请求的玩家
     *
     * @param reason 拒绝原因
     */
    public void rejectProcess(Reason reason) {
        updateStatusWhenNotProcessed(ContextStatus.REJECTED);
        completeHookFuture(HookResponse.reject(reason));
    }

    void updateStatusWhenNotProcessed(ContextStatus newStatus) {
        if (status == ContextStatus.NEW || status == ContextStatus.DEFERRED) {
            status = newStatus;
        } else {
            throw new IllegalStateException("request already processed");
        }
    }

    void completeHookFuture(HookResponse<T> response) {
        req.setReadOnly();
        future.complete(response);
    }
}

==> game-hook-base-1.1-sources.jar: cn/leancloud/play/hook/context/AbstractOperationContext.java <==
package cn.leancloud.play.hook.context;

import cn.leancloud.play.hook.HookResponse;
import cn.leancloud.play.hook.HookedRoom;
import cn.leancloud.play.hook.Reason;
import cn.leancloud.play.hook.request.RoomRequest;

import java.util.Objects;
import java.util.concurrent.CompletableFuture;

abstract class AbstractOperationContext<T extends RoomRequest> implements Context<T> {
    private final CompletableFuture<HookResponse<T>> future;
    private final T req;
    private final HookedRoom hookedRoom;

    private ContextStatus status;

    AbstractOperationContext(T req, HookedRoom hookedRoom, CompletableFuture<HookResponse<T>> future) {
        Objects.requireNonNull(req);
        Objects.requireNonNull(future);

        this.req = req;
        this.future = future;
        this.status = ContextStatus.NEW;
        this.hookedRoom = hookedRoom;
    }

    @Override
    public ContextStatus getStatus() {
        return status;
    }

    @Override
    public T getRequest() {
        return req;
    }

    @Override
    public HookedRoom getHookedRoom() {
        return hookedRoom;
    }

    @Override
    public void continueProcess() {
        updateStatusWhenNotProcessed(ContextStatus.CONTINUED);
        completeHookFuture(HookResponse.success(req));
    }

    @Override
    public void rejectProcess(Reason reason) {
        updateStatusWhenNotProcessed(ContextStatus.REJECTED);
        completeHookFuture(HookResponse.reject(reason));
    }

    void updateStatusWhenNotProcessed(ContextStatus newStatus) {
        if (status == ContextStatus.NEW || status == ContextStatus.DEFERRED) {
            status = newStatus;
        } else {
            throw new IllegalStateException("request already processed");
        }
    }

    void completeHookFuture(HookResponse<T> response) {
        req.setReadOnly();
        future.complete(response);
    }
}
